package Crop::HTTP::Request::PSGI;
use base qw/ Crop::Object /;

=begin nd
Class: Crop::HTTP::Request::PSGI
	PSGI specific request.
	
	NOT IMPLEMENTED.
=cut

use v5.14;
use warnings;

use Crop::Error;

sub new {
	warn __PACKAGE__ . ' NOT IMPLEMENTED';
}

1;
